@@ -88,9 +88,9 @@ void close_notification(struct mako_notification *notif,
 	enum mako_notification_close_reason reason,
 	bool add_to_history);
 void close_group_notifications(struct mako_notification *notif,
-	enum mako_notification_close_reason reason);
+	enum mako_notification_close_reason reason, bool add_to_history);
 void close_all_notifications(struct mako_state *state,
-	enum mako_notification_close_reason reason);
+	enum mako_notification_close_reason reason, bool add_to_history);
 char *format_hidden_text(char variable, bool *markup, void *data);
 char *format_notif_text(char variable, bool *markup, void *data);
 size_t format_text(const char *format, char *buf, mako_format_func_t func, void *data);
