@@ -32,11 +32,14 @@ static const char usage[] =
 	"                                      meaning as in CSS.\n"
 	"      --border-size <px>              Border size.\n"
 	"      --border-color <color>          Border color.\n"
-	"      --border-radius <px>            Corner radius\n"
+	"      --border-radius <px>[,<px>...]  Corner radius, comma separated.\n"
+	"                                      Up to four values, with the same\n"
+	"                                      meaning as in CSS.\n"
 	"      --progress-color <color>        Progress indicator color.\n"
 	"      --icons <0|1>                   Show icons in notifications.\n"
 	"      --icon-path <path>[:<path>...]  Icon search path, colon delimited.\n"
 	"      --max-icon-size <px>            Set max size of icons.\n"
+	"      --icon-border-radius <px>       Icon's corner radius.\n"
 	"      --markup <0|1>                  Enable/disable markup.\n"
 	"      --actions <0|1>                 Enable/disable application action\n"
 	"                                      execution.\n"
