@@ -1,4 +1,3 @@
-#define _POSIX_C_SOURCE 200809L
 #include <errno.h>
 #include <getopt.h>
 #include <libgen.h>
@@ -14,7 +13,7 @@
 #include "types.h"
 
 
-const char VALID_FORMAT_SPECIFIERS[] = "%asbhtg";
+const char VALID_FORMAT_SPECIFIERS[] = "%asbhtgi";
 
 
 bool parse_boolean(const char *string, bool *out) {
